# -*- coding: utf-8 -*-
"""
Created on Sat Jan 11 14:57:41 2020

@author: mendez
"""

# Exercise 1 :

import numpy as np # Numpy is for scientific computing
import matplotlib.pyplot as plt  # Matplotlib is for plotting.
import shutil # To copy the figures in the Latex folder


# Define a delta function

def delta_F(n,k):  
    """
    This function defines the delta function in a 1D domain.
          
    :param n: The signal's domain
    :param k: The location of the delta function
    """
     # check if k is within n
    if np.any(n[:] == k):
     delta_f=np.zeros(n.shape) 
     delta_f[n==k]=1
     return delta_f
    else:
     raise ValueError("k is not in n") 
     return -1
    
    
# Define a Step function
def step_F(n,k):  
    """
    This function defines the step function in a 1D domain.
          
    :param n: The signal's domain
    :param k: The location of the step function
    """
     # check if k is within n
    if np.any(n[:] == k):
     u_S=np.zeros(n.shape) 
     u_S[n>=k]=1
     return u_S
    else:
     raise ValueError("k is not in n") 
     return -1

# Define a box function
    
def box_F(n,k1,k2):  
    """
    This function defines the box function in a 1D domain.
          
    :param n: The signal's domain
    :param k1: The location of the first edge
    :param k2: The location of the second edge
    
    """
     # check if k is within n
    if np.any(n[:] == k1) and np.any(n[:] == k2):
     u_S1=np.zeros(n.shape); u_S1[n>=k1]=1 # Step function 1
     u_S2=np.zeros(n.shape); u_S2[n>=k2]=1 # Step function 2
     box_S=u_S1-u_S2 # Box function Definition
     return box_S
    else:
     raise ValueError(" box outside domain n") 
     return -1
    

# plot a box function 2,6
n=np.arange(-3,8,1) # Domain for the plot
BOX=box_F(n,2,6)

fig, ax = plt.subplots(figsize=(4, 3)) # This creates the figure
# Or you can plot it as streamlines
plt.stem(n,BOX,use_line_collection=True)
plt.rc('text', usetex=True)      
plt.rc('font', family='serif')
plt.rc('xtick',labelsize=13)
plt.rc('ytick',labelsize=13)
#ax.set_aspect('equal') # Set equal aspect ratio
ax.set_xlabel('$k$',fontsize=14)
ax.set_title('$u_{\mathcal{B}{2,6}}$',fontsize=18)
ZZ=np.where(n==0)[0][0] # find the zero to remove from axis
ax.set_xticks(np.concatenate((n[0:ZZ],n[ZZ+1::])))
ax.set_ylim([-0.1,1.1])
ax.set_xlim(n.min(),n.max())
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
ax.spines['left'].set_position('zero')
ax.spines['bottom'].set_position('zero')
ax.set_yticks([0.5,1])
Name='Box_Example.pdf'
plt.tight_layout()
plt.savefig(Name, dpi=200) 
shutil.copyfile(Name,'../Figures/'+Name) #  copy file to latex directory

# Exercise 1
n=np.arange(0,380,1) # Domain for the plot
n_t=320 # Number of point
f_s=100; dt=1/f_s # Compute Delta t
t_k=n*dt # Time domain axis
BOX_OBS=box_F(n,0,n_t)
y=np.exp(-0.2*t_k)*np.cos(2*np.pi*t_k)*BOX_OBS # discretize signal
y_n=np.exp(-0.2*dt)**n*np.cos(2*dt*np.pi*n)*BOX_OBS # fully discrete signal
plt.close(fig='all')

fig, ax = plt.subplots(figsize=(4, 3)) # This creates the figure
plt.plot(y,'ko--',markerfacecolor=[0,0,0],markersize=4)
#plt.plot(y_n,'rs') # Check that the two are equivalent
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
ax.set_xlabel('$k$',fontsize=14)
ax.set_title('$(0.998)^k\,\cos(0.02 \pi k)$',fontsize=14)
plt.tight_layout()
Name='Ex1.pdf'
plt.savefig(Name, dpi=200) 
shutil.copyfile(Name,'../Figures/'+Name) #  copy file to latex directory

# Check when a discrete sinusoidal is periodic:
f_d=0.01
n=np.arange(-100,100,1)
plt.plot(np.cos(2*np.pi*f_d*n))




