# -*- coding: utf-8 -*-
"""
Created on Tue Jan 28 08:58:40 2020

@author: mendez
"""

import numpy as np # Numpy is for scientific computing
import matplotlib.pyplot as plt  # Matplotlib is for plotting.
import shutil # To copy the figures in the Latex folder


# the LCCDE of a discrete system of order 2 is given.
# the coefficients are
a_1=-1.1262256578555037 # From previous exercise
a_2=0.513417119032592 # From previous exercise
K=(1+a_1+a_2)

# prepare the discrete frequency (angle) domain
n_f=2000 # number of frequencies
theta=np.linspace(0,np.pi,n_f) # frequency domain
# The Transfer function in the discrete domain is
H=K/(1+a_1*np.exp(-1j*theta)+a_2*np.exp(-2j*theta)) 
# If we set a_1=0, but keep the same static gain:
H_1=(1+a_2)/(1+a_2*np.exp(-2j*theta))
# If we set a_2=0, but keep the same static gain:
H_2=(1+a_1)/(1+a_1*np.exp(-1j*theta))



fig, ax = plt.subplots(figsize=(4, 3)) # This creates the figure
plt.rc('text', usetex=True)      
plt.rc('font', family='serif')
plt.rc('xtick',labelsize=13)
plt.rc('ytick',labelsize=13)
plt.plot(theta,np.absolute(H),'k-',label='$H(f)$')
plt.plot(theta,np.absolute(H_1),'b--',label='$H(f,a_1=0)$')
plt.plot(theta,np.absolute(H_2),'r-.',label='$H(f,a_2=0)$')
#ax.set_aspect('equal') # Set equal aspect ratio
plt.xlabel('$\\theta_n$',fontsize=14)
ax.set_xlim(0,np.pi)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
ax.set_ylim([0,3.5])
plt.legend(loc='upper right')
Name='H1H2H3.pdf'
plt.tight_layout()
plt.savefig(Name, dpi=200) 
#shutil.copyfile(Name,'../Figures/'+Name) #  copy file to latex directory


# Now we proceed with the derivation of the High pass filter.


# It is interesting to see that the complementary transfer function
# does not give complementary amplitude function
# Complementary Z Transfer function
K=(1+a_1+a_2)
H_C=((1-K)+a_1*np.exp(-1j*theta)+a_2*np.exp(-2j*theta))/(1+a_1*np.exp(-1j*theta)+a_2*np.exp(-2j*theta))
#
#fig, ax = plt.subplots(figsize=(4, 3)) # This creates the figure
#plt.rc('text', usetex=True)      
#plt.rc('font', family='serif')
#plt.rc('xtick',labelsize=13)
#plt.rc('ytick',labelsize=13)
#plt.plot(theta,np.absolute(H)+np.absolute(H_C),'k-',label='$H_C(f)$')
## This is due to the phase delay.

## The above transfer function can also be written in terms of sine/cosing
## transforms:
#A=1+a_1*np.cos(theta)+a_2*np.cos(theta)
#B=a_1*np.sin(theta)+a_2*np.sin(theta)
#H_sc=(K)/(A-1j*B)
## The modulus of this is 
#H_L_abs=K/np.sqrt(A**2+B**2)
## The argument of this is 
#H_L_arg=np.arctan(B/A)
## The modulus of the complementary is
#H_H_abs=(np.sqrt(A**2+B**2)-K)/(np.sqrt(A**2+B**2))
### What if we CHOOSE to keep the same phase response?
##H_H=H_H_abs*np.exp(1j*H_H_abs) 
#
K=(1+a_1+a_2)
H_C=((1-K)+a_1*np.exp(-1j*theta)+a_2*np.exp(-2j*theta))/(1+a_1*np.exp(-1j*theta)+a_2*np.exp(-2j*theta))

K=(1+a_1)
H_C_1=((1-K)+a_1*np.exp(-1j*theta))/(1+a_1*np.exp(-1j*theta))

K=(1+a_2)
H_C_2=((1-K)+a_2*np.exp(-2j*theta))/(1+a_2*np.exp(-2j*theta))


fig, ax = plt.subplots(figsize=(4, 3)) # This creates the figure
plt.rc('text', usetex=True)      
plt.rc('font', family='serif')
plt.rc('xtick',labelsize=13)
plt.rc('ytick',labelsize=13)
plt.plot(theta,np.absolute(H_C),'k-',label='$H_C(f)$')
plt.plot(theta,np.absolute(H_C_1),'b--',label='$H_C(f,a_1=0)$')
plt.plot(theta,np.absolute(H_C_2),'r-.',label='$H_C(f,a_2=0)$')

# This is due to the phase delay.
#ax.set_aspect('equal') # Set equal aspect ratio
plt.xlabel('$\\theta_n$',fontsize=14)
ax.set_xlim(0,np.pi)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
ax.set_ylim([0,2.2])
plt.legend(loc='upper right')
Name='H_Complement.pdf'
plt.tight_layout()
plt.savefig(Name, dpi=200) 
#shutil.copyfile(Name,'../Figures/'+Name) #  copy file to latex directory








