# -*- coding: utf-8 -*-
"""
Created on Thu Feb  4 17:48:10 2021

@author: mendez
"""

import numpy as np
import matplotlib.pyplot as plt

#%% Introduction
# We want to study and predict the response of a system with
# input u and output y.

# We will make the identification/regression in a time span 0-5 
# and we will make prediction in the time span 0-10.

# This is the time span over which we will do the regression
n_t=2000
t_s=np.linspace(0,5,n_t) # Time axis
u_i_s= 1*np.sin(8*np.pi*t_s)+3*np.exp(-(t_s-3.3)**2/2) # Input
plt.plot(t_s,u_i_s)

# This is the time span over which we will do the predictions
t_ss=np.linspace(0,10,n_t) # Time axis
u_i_ss= 1*np.sin(8*np.pi*t_ss)+3*np.exp(-(t_ss-3.3)**2/2)# Input
plt.plot(t_ss,u_i_ss,'--')

#%% Definition of the system
# We take a 3 orders system, with white noise
a=np.array([0,-0.5,0]) # Feedback coefficients
b=np.array([2,1.8,-1.2]) # Feedforward coefficients


#%% Functions needed in the exercise
# This function simulates a stochastic LTI system with
# white noise in [0, Eps]

def Solve_LTI_3(a,b,t,u,Eps):
  y=np.zeros((int(len(t)),1))
  for k in range(2,n_t): 
   epsilon=np.random.uniform(0,1,1)   
   y[k]=b[0]*u[k]+b[1]*u[k-1]+b[2]*u[k-2]+a[1]*y[k-1]+a[2]*y[k-2] \
       +Eps*epsilon[0]
  return y

# This function implements the regression with L2 Penalty
# and regularization alpha. This first crude implementation
# only works for a system with order 3

def AR_Reg_3_RIDGE(u,y,alpha):
  # Perform the Auto Regression to find the coefficients a, b
  N_S=2 # Number of shifts
  n_t=int(len(u))
  u_l0=u[N_S:n_t-N_S]
  u_l1=u[N_S+1:n_t-N_S+1]
  u_l2=u[N_S+2:n_t-N_S+2]
  y_l0=y[N_S:n_t-N_S]
  y_l1=y[N_S+1:n_t-N_S+1]
  y_l2=y[N_S+2:n_t-N_S+2]
  A=np.column_stack((u_l0,u_l1,u_l2,y_l1,y_l2))
  X_C=np.linalg.inv(A.T.dot(A) + alpha * np.eye(A.shape[1])).dot(A.T).dot(y_l0)
  b=X_C[0:3]; a=np.zeros(b.shape); a[1]=X_C[3];a[2]=X_C[4]  
  Err=np.linalg.norm(y_l0-A.dot(X_C))/np.linalg.norm(y_l0)
  return a, b, Err


#%% Exercise Solution: FIT
# Here we simulate one instance of the system output.
y_s=Solve_LTI_3(a,b,t_s,u_i_s,2)
plt.plot(t_s,y_s)
# Note that if you run this 100 times, 
# you get 100 different results. These look similar nevertheless

# Here we do the regression
a_r,b_r,Err=AR_Reg_3_RIDGE(u_i_s,y_s,2)
print(Err)
# Observe that the coefficients obtained are very different!
# Check that the identified system gives a similar output
y_s_T=Solve_LTI_3(a_r,b_r,t_s,u_i_s,0)
plt.plot(t_s,y_s) # Actual DATA (used for regression)
plt.plot(t_s,y_s_T,'r--') # Regression

#%% Test the Prediction
# Here we see how the detected system compares in the prediction

y_ss_r=Solve_LTI_3(a,b,t_ss,u_i_ss,2)
plt.plot(t_ss,y_ss_r,'b',label='Reality')

y_ss_p=Solve_LTI_3(a_r,b_r,t_ss,u_i_ss,0)
plt.plot(t_ss,y_ss_p,'r-',label='Prediction')

plt.legend()

